#ifndef COTEXTREEINTERFACE_H
#define COTEXTREEINTERFACE_H

#include <Object.h>

#include "PrimitiveInterface.h"
#include "CoTeXNode.h"
#include "TextureInterface.h"
#include "OperatorInterface.h"
#include "HyperFunable.h"

/// Interface for Constructive texturing trees (ctt).
/**
 * Provide one main service : the definition of the Object's attributes.
 * This interface defined the add/remove opreations. The inherited classes
 * have just to specified their HyperFun specifications.
 * Nevertheless, they can re-define the mains methods for adapting to a specific 
 * situation.
 */
class CoTeXTreeInterface : public HyperFunable{
public:
  
  /// Map for quering the CoTeXTree of a specified Object.
  typedef QMap<Object*,CoTeXTreeInterface*> CoTeXTreeMap;
  
  /// The base constructor.
  /**
   * @param attribute the base action of the tree while be to affect surfels 
   *                  with this attribute.
   */
  CoTeXTreeInterface (AttributeInterface * attribute);

  CoTeXTreeInterface (const CoTeXTreeInterface & tree);
  
  virtual ~CoTeXTreeInterface ();
  
  inline CoTeXNode * getRoot () const { return root;}
  void  setRoot (CoTeXNode * newRoot);
  inline bool isEmpty () { return (root == NULL);}
  
  /// The tree contains p only if the root appear in the hierarchy of p
  /**
   * Root is not necessary the first element of the hierarchie, because we can 
   * imagine a composition of tree in future extensions.
   */
  bool contains (PrimitiveInterface * p) const;

  /// Merging a primitive in the tree.
  /** 
   * Add a primitive to the current node of the tree, joining them through 
   * the argument operator.
   * @param p the primitive added.
   * @param op the operator for the link with the current node. Destroyed 
   *           if the tree was empty.
   */
  virtual void addPrimitive (PrimitiveInterface * p, OperatorInterface * op);

  /// Removing a primitive from the tree.
  /**
   * The primitive's father is automatically destroyed, replaced by
   * the brother of the removed primtive (as son of the grand father, 
   * or root if there's not).
   * @param p the primitive removed, the tree DOES NOT delete it. 
   * @return PrimtiveInterface* the primtive removed : p.
   */
  virtual PrimitiveInterface * removePrimitive (PrimitiveInterface * p);

  /// Compute attribute for each surfel.
  /**
   * Each attribute computed is affected to its surfel.
   * This action is defined by the method <code>affecSurfel</code>
   * of the class <code>AttributeInterface</code>.
   *  Primitives are expressed in the local mark of the object 
   * (thanks to surfels expressed themselves in the local mark).
   *  This method is virtual, so classes which inherit from this
   * can define other type of applications.
   * @param collection the surfels modified.
   */
  virtual void apply (SurfelCollection * collection);


  /// The same, but on a whole object.
  void apply (Object * object);

protected:
  
  inline CoTeXNode * getCurrent () const { return current;}
  inline void  setCurrent (CoTeXNode * newCurrent) { current = newCurrent;}
  inline AttributeInterface * getDefaultAttribute () const { return defaultAttribute;}
  void setDefaultAttribute (AttributeInterface * newDefault);


private:

  CoTeXNode * root; 
  CoTeXNode * current; ///< by default, the last added
  AttributeInterface * defaultAttribute;
};

#endif // COTEXTREEINTERFACE_H
